const std = @import("std");
const minifb_c = @cImport({
    @cInclude("MiniFB.h");
    @cInclude("MiniFB_enums.h");
});
const testing = std.testing;

pub fn rgb(r: u8, g: u8, b: u8) u32 {
    return @intCast(u32, r) << 16 | @intCast(u32, g) << 8 | b;
}

pub const Window = struct{
    cwin: *minifb_c.mfb_window,

    pub const UpdateError = error {
        InvalidWindow,
        InvalidBuffer,
        InternalError,
    };

    pub const State = enum {ok, exit};

    pub fn open(title: [*:0]const u8, width: u32, height: u32) !Window {
        const cwin: ?*minifb_c.mfb_window = minifb_c.mfb_open(@as([*c]const u8, title), width, height);
        if (cwin) |value| {
            return Window {.cwin=value};
        } else {
            return error.ItBroke;
        }
        
    }

    pub fn waitSync(self: Window) bool {
        return minifb_c.mfb_wait_sync(self.cwin);
    }

    pub fn update(self:Window, buffer: []u32) UpdateError!State {
        const rawState = minifb_c.mfb_update(self.cwin, buffer.ptr);
        switch (rawState) {
            .STATE_OK => return State.ok,
            .STATE_EXIT => return State.exit,
            .STATE_INVALID_WINDOW => return UpdateError.InvalidWindow,
            .STATE_INVALID_BUFFER => return UpdateError.InvalidBuffer,
            else => return UpdateError.InternalError,
        }
    }
};

pub fn getTargetFPS() u32 {
    return minifb_c.mfb_get_target_fps();
}

pub fn setTargetFPS(fps: u32) void {
    minifb_c.mfb_set_target_fps(fps);
}

// TODO Figure out how to run this once I have Internet access.
test "set and get target FPS" {
    const max = 40;
    var fps: u32 = 30;
    while (fps < max) {
        setTargetFPS(fps);
        try std.testing.expectEqual(fps, getTargetFPS());
    }
}
